@@ -1,6 +1,8 @@
 import os, errno, importlib, uuid, threading
 
 from django.core.files import base, temp
+from django.utils.deconstruct import deconstructible
+from django.utils.encoding import force_text
 from django.utils.six.moves import queue as six_queue
 
 __all__ = [
@@ -18,7 +20,8 @@ class ProcessingError(Exception):
 
 class NamedTemporaryFile(base.File):
     """This class is required for FileStorage to make an attempt in moving the
-    file, instead of copying it by chunks in memory
+    file, instead of copying it by chunks in memory. Borrowed implementation from
+    `django.core.files.uploadedfile.TemporaryUploadedFile`
 
     """
     def __init__(self, **kwargs):
@@ -32,18 +35,14 @@ class NamedTemporaryFile(base.File):
         return self.file.name
 
     def close(self):
-        # caching the size before closing for proper file moving and saving
         self._size = self._get_size()
         try:
             return self.file.close()
         except OSError as e:
             if e.errno != errno.ENOENT:
-                # Means the file was moved or deleted before the tempfile
-                # could unlink it.  Still sets self.file.close_called and
-                # calls self.file.file.close() before the exception
                 raise
 
-
+@deconstructible
 class UploadTo(object):
     """This is an upload filename generator to be used to create a function to be
     passed as an ``upload_to`` keyword argument to the ``models.FileField`` and
@@ -76,27 +75,24 @@ class UploadTo(object):
     :keyword str field_name: if ``None`` will skip insertion of a field_name
     (Default), otherwise ``field_name`` string will be inserted before the subfolder.
 
-    :keyword bool generate: if set to ``True`` will replace the ``name`` portion
-    of the file with a string generated by ``generator`` function.
-
     :keyword function generator: function that will generate a new name portion
-    of the file in case if ``generate`` is set to ``True``. Default is
-    :func:`uuid.uuid1`.
+    of the file in case if it is set to ``True`` default generator :func:`uuid.uuid1`
+    will be used.
 
-    :keyword function parent_pk_getter: function that takes model instance and
-    returns pk of the parent
+    :keyword str parent_field_name: name of the ForeignKey or OneToOneField, that 
+    is considered a parent for this field's model. Set to an empty string to ignore parent_pk
 
     """
 
     def __init__(self, basefolder=None, subfolder=None, filename=None, name=None, ext=None,
-                 app_label=None, model_name=None, parent_pk_getter=None, field_name=None, 
-                 generate=False, generator=uuid.uuid1):
+                 app_label=None, model_name=None, parent_field_name=None, field_name=None, 
+                 generator=None):
         assert filename is None or name is None, \
             "Cannot have 'filename' and 'name' specified at the same time." 
-        assert not generate or (filename is None and name is None), \
+        assert generator is None or (filename is None and name is None), \
             "Cannot specify a name and enforce name generation"
-        assert not generate or callable(generator), \
-            "Supply a function for a generator"
+        assert generator is None or generator is True or callable(generator), \
+            "Supply a function for a generator or set it to True to use default generator."
         self.basefolder = basefolder
         self.subfolder = subfolder
         self.filename = filename
@@ -104,11 +100,23 @@ class UploadTo(object):
         self.ext = ext
         self.app_label = app_label
         self.model_name = model_name
-        self.parent_pk_getter = parent_pk_getter if callable(parent_pk_getter) else None
+        self.parent_field_name = parent_field_name
         self.field_name = field_name
-        self.generate = generate
-        self.generator = generator
+        self.generator = uuid.uuid1 if generator is True else generator
 
+    def __eq__(self, other):
+        return (type(self) is type(other) and
+                self.basefolder == other.basefolder and
+                self.subfolder == other.subfolder and
+                self.filename == other.filename and
+                self.name == other.name and
+                self.ext == other.ext and
+                self.app_label == other.app_label and
+                self.model_name == other.model_name and
+                self.parent_field_name == other.parent_field_name and
+                self.field_name == other.field_name and
+                self.generator is other.generator)
+        
     def __call__(self, instance, filename):
         structure = []
         if self.basefolder is not None:
@@ -121,32 +129,47 @@ class UploadTo(object):
             structure.append(instance._meta.model_name)
         elif self.model_name:
             structure.append(self.model_name)
-        if self.parent_pk_getter is not None:
-            structure.append(str(self.parent_pk_getter(instance)))
+        parent_pk = self.get_parent_pk(instance)
+        if parent_pk is not None:
+            structure.append(parent_pk)
         if self.subfolder is not None:
             structure.append(self.subfolder)
         if instance.pk is not None:
-            structure.append(str(instance.pk))
+            structure.append(force_text(instance.pk))
         if self.field_name is not None:
             structure.append(self.field_name)
+        structure.append(self.get_filename(filename))
+        return os.path.join(*structure)
+
+    def get_filename(self, filename):
         if self.filename is not None:
             filename = self.filename
         else:
             name, ext = os.path.splitext(filename)
             if ext: # remove a dot, but only if there is an extension
                 ext = ext[1:]
-            if self.generate:
+            if self.generator is not None:
                 name = self.generator()
             elif self.name is not None:
                 name = self.name
             if self.ext is not None:
                 ext = self.ext
             if ext:
-                filename = "%s.%s" % (name, ext.lower())
+                filename = "%s.%s" % (name, ext)
             else: 
                 filename = name
-        structure.append(filename)
-        return os.path.join(*structure)
+        return filename
+
+    def get_parent_pk(self, instance):
+        parent_field_name = None
+        if self.parent_field_name is None:
+            parent_field_name = getattr(instance, 'parent_field_name', None)
+        elif self.parent_field_name:
+            parent_field_name = self.parent_field_name
+        if parent_field_name is not None:
+            parent = getattr(instance, parent_field_name)
+            if parent is not None:
+                return force_text(parent.pk)
 
 
 class AsynchronousFileReader(threading.Thread):
